/**
 * 购物车操作
 */
import store from '@/store/index'
import { SET_CARTDATA, SET_CARTDATA_INIT, SET_REDUCTIONS } from '@/store/mutation-types'
import { numSubtract, numMultiply, numDivide, getReductionRuleInfo, numAdd } from '@/common/utils'
import { REDUCTION_MODE } from '@/common/constant'
import { GetCart } from '@/service/cart'

const ClassCart = function() {}
ClassCart.prototype.cartToStore = function(cart) {
	store.commit(SET_CARTDATA, cart)
}

ClassCart.prototype.reductionsToStore = function(reductions) {
	store.commit(SET_REDUCTIONS, reductions)
}

// 初始化购物车接口返回数据，暂未排序
ClassCart.prototype.initGroup = function(data) {
	// console.log('initGroup', data)
	// 数据结构 [{组1, 组2, 组3}]
	const groupList = []
	// 满额优惠
	if (data.reductions) {
		data.reductions.forEach(item => {
			item.customType = 'reductions'
			groupList.push(item)
		})
	}
	
	// 第二件半价
	if (data.rebates) {
		data.rebates.forEach(item => {
			item.customType = 'rebates'
			groupList.push(item)
		})
	}
	
	// 加价购
	if (data.addOnItems) {
		data.addOnItems.forEach(item => {
			item.customType = 'addOnItems'
			groupList.push(item)
		})
	}
	
	if (data.items) {
		// 常规
		const validList = data.items.filter(p => p.status === 1);
		if (validList.length) {
			groupList.push({
				customType: 'valid',
				items: validList
			})
		}
		// 失效
		const invalid = data.items.filter(p => p.status !== 1)
		if (invalid.length) {
			groupList.push({
				customType: 'invalid',
				items: invalid
			})
		}
	}
	return groupList
}

// 初始化分组数据，
ClassCart.prototype.initProductData = function({groupList, needSelect, defaultSelect = []}) {
	groupList.forEach(groupData => {
		groupData.items.forEach(p => {
			// 限购商品购物车最大应该取限购数
			if (p.limitId && p.limitValue > 0) {
				p.stock = p.limitValue
				if (p.quantity > p.limitValue) {
					p.quantity = p.limitValue
				}
			}
			
			// 满减的选中逻辑
			if (groupData.customType === 'reductions') {
				// 首次加入购物车需要默认勾选，其他操作不需要
				if (!needSelect) {
					// 默认勾选
					const defaultItem = defaultSelect.find((item) => {
						return item === p.skuId
					})
					p.checked = false
					p.stockError = false
					if (p.createTime && defaultSelect.length == 0) {
						const createtTime = new Date(p.createTime.replace(/-/g, '/')).getTime()
						const nowTime = Date.now()
						// 小于10分钟，默认选中
						if (parseInt((nowTime - createtTime) / 1000 / 60) < 10) {
							p.checked = true
						}
					}
					if (defaultItem) {
						p.checked = defaultItem ? true : false
					}
				}
			} else {
				// 常规商品的选中逻辑
				const item = defaultSelect.find((item) => {
					return item === p.skuId
				})
				p.checked = false
				p.stockError = false
				if (p.createTime && defaultSelect.length == 0 && groupData.customType !== 'invalid') {
					const createtTime = new Date(p.createTime.replace(/-/g, '/')).getTime()
					const nowTime = Date.now()
					// 小于10分钟，默认选中
					if (parseInt((nowTime - createtTime) / 1000 / 60) < 10) {
						p.checked = true
					}
				}
				if (item) {
					p.checked = item ? true : false
				}
			}
			
		})
	})
	return groupList
}

ClassCart.prototype.set = function(data) {
	const cartData = {}
	if (!data) {
		return
	}
	// 格式化数据到list
	Object.keys(data).map(key => {
		const item = data[key]
		item.forEach(subItem => {
			if (subItem.items) {
				subItem.items.forEach(p => {
					cartData[p.skuId] = p
				})
			} else {
				cartData[subItem.skuId] = subItem
			}
		})
	})
	// 每次设置都先清空原有数据
	store.state.cartData = {}
	// console.log('cartData', cartData)
	store.commit(SET_CARTDATA_INIT, true)
	this.cartToStore(cartData)
	// this.reductionsToStore(reductions)
}

ClassCart.prototype.get = function(skuId) {
	return store.state.cartData[skuId]
}

ClassCart.prototype.add = function(params) {
	if (params && params.skuId) {
		const cart = store.state.cartData
		cart[params.skuId] = {
			productId: params.id,
			skuId: params.skuId,
			status: 1,
			price: params.price,
			quantity: params.quantity
		}
		this.cartToStore(cart)
	}
}

ClassCart.prototype.del = function(params) {
	// 如果是多个删除
	if (Object.prototype.toString.call(params) === '[object Array]') {
		const cart = store.state.cartData
		params.forEach(p => {
			delete cart[p.skuId]
		})
	} else {
		delete cart[params.skuId]
	}
	this.cartToStore(cart)
}

ClassCart.prototype.updateItemTo = function(params) {
	const skuId = params.skuId
	// 存在更新，不存在添加
	if (this.get(skuId)) {
		const cart = store.state.cartData
		cart[skuId].quantity = params.quantity
		this.cartToStore(cart)
	} else {
		this.add(params)
	}
}

ClassCart.prototype.updateItem = function(params) {
	// 本地还没有进行数据库请求过，先请求
	if (!store.state.cartDataInit) {
		// console.log('没有加载')
		GetCart({showLoading: false}).then(res => {
			if (res.success) {
				store.commit(SET_CARTDATA_INIT, true)
				this.set(res.data)
			}
		})
	} else {
		// console.log('加载过了')
		this.updateItemTo(params)
	}
}

/**
 * 组合规则数据
 * @param {Object} rule
 * @param {Object} money
 * @param {Object} next
 * @param {Object} rules
 * @param {Object} mode
 */
ClassCart.prototype.setNextReductionRule = function(rule, money, quantity, next, rules, mode) {
	const reduction = {}
	reduction.reduceMoney = rule.rebate ? rule.rebate : rule.amount
	reduction.isRebate = rule.rebate > 0
	reduction.diffMoney = this.getDiffMoney(rules, money, mode)
	reduction.diffQuantity = this.getDiffMoney(rules, quantity, mode)
	getReductionRuleInfo(rule, mode === REDUCTION_MODE.LOOP, money, quantity, next, reduction)
	return reduction
}


/**
 * 通过金额匹配规则
 * @param {Object} rules
 * @param {Object} money
 * @param {Object} mode 
 */
ClassCart.prototype.getReductionsInfo = function(rules, money, quantity, mode, type) {
	// 阶梯
	let currentInfo = {}
	let nextInfo = {}
	let currentIndex = null
	
	let currentRule = null
	let nextRule = null
	if (mode === REDUCTION_MODE.STEP) {
		rules.forEach((item, index) => {
			if ((type === 1 && money >= item.limit) || (type === 2 && quantity >= item.limit)) {
				currentIndex = index
			}
		})
		// 一档都没有满足
		if (currentIndex === null) {
			nextRule = rules[0]	
			nextInfo = Object.assign(nextInfo, this.setNextReductionRule(nextRule, money, quantity, true, rules, mode))
			
			currentInfo.profits = []
			currentInfo.qProfits = []
			currentInfo.tags = []
		} else {
			// 至少满足最低档
			let nextRule = rules[currentIndex + 1]
			if (nextRule) {
				nextInfo = Object.assign(nextInfo, this.setNextReductionRule(nextRule, money, quantity, true, rules, mode))
			} else {
				nextInfo.diffMoney = 0
				nextInfo.diffQuantity = 0
				nextInfo.reduceMoney = 0
				nextInfo.profits = []
				nextInfo.qProfits = []
				nextInfo.tags = []
			}
			
			currentRule = rules[currentIndex]
			if (currentRule) {
				let isMax = false
				if (currentIndex === rules.length - 1) {
					isMax = true
				}
				currentInfo = Object.assign(currentInfo, this.setNextReductionRule(currentRule, money, quantity, false, rules, mode))
				currentInfo.isMax = isMax
			}
		}
	}
	
	// 循环
	if (mode === REDUCTION_MODE.LOOP) {
		let rule = rules[0]
		let currentIndex = 0
		let nextIndex = 0
		
		if (type === 1) {
			currentIndex = Math.floor(numDivide(money, rule.limit))
			nextIndex = Math.ceil(numDivide(money, rule.limit))
			// 如果发现刚好是整数，向后取
			if (money % rule.limit === 0) {
				nextIndex += 1
			}
		} else {
			currentIndex = Math.floor(numDivide(quantity, rule.limit))
			nextIndex = Math.ceil(numDivide(quantity, rule.limit))
			// 如果发现刚好是整数，向后取
			if (quantity % rule.limit === 0) {
				nextIndex += 1
			}
		}
		
		nextInfo.reduceMoney = rule.amount * nextIndex
		nextInfo.isRebate = false // 循环满减没有打折
		nextInfo.diffMoney = this.getDiffMoney(rules, money, mode)
		nextInfo.diffQuantity = this.getDiffMoney(rules, quantity, mode)
		nextInfo = Object.assign(nextInfo, this.setNextReductionRule(rule, money, quantity, true, rules, mode))
		// 至少满足一档了
		if (currentIndex > 0) {
			currentInfo.reduceMoney = rule.amount * currentIndex
			currentInfo.isRebate = false
			currentInfo.diffMoney = this.getDiffMoney(rules, money, mode)
			currentInfo.diffQuantity = this.getDiffMoney(rules, quantity, mode)
			currentInfo = Object.assign(currentInfo, this.setNextReductionRule(rule, money, quantity, false, rules, mode))
			let isMax = false
			if (type === 1) {
				isMax = Math.floor(numDivide(money, rule.limit)) >= 100 // 元
			} else {
				isMax = Math.floor(numDivide(quantity, rule.limit)) >= 100 // 件
			}
			currentInfo.isMax = isMax // 循环满减无最大值
		} else {
			currentInfo.profits = []
			currentInfo.qProfits = []
			currentInfo.tags = []
		}
	}
	
	return {
		nextInfo,
		currentInfo
	}
}

/**
 * 通过金额匹配第二件半价
 * @param {Object} rules
 * @param {Object} money
 * @param {Object} mode 
 */
ClassCart.prototype.getrebatesInfo = function(rebate) {
	let unit = `${rebate.discount}折`
	if (rebate.discount === 5) {
		unit = `半价`
	}
	if (rebate.discount === 0) {
		unit = '0元'
	}
	let current = 0
	rebate.items.forEach(p => {
		current = numAdd(current, p.quantity)
	})
	let page = Math.floor(numDivide(current, rebate.quantity))
	let isMax =  page >= 100 // 元
	
	let remainder = Math.ceil(current % rebate.quantity)
	let nextNumber = numSubtract(rebate.quantity, remainder)
	return {
		isMax,
		nextNumber,
		discount: rebate.discount,
		quantity: rebate.quantity,
		rebateId: rebate.rebateId,
		unit
	}
}
/**
 * 通过金额匹配加价购
 */
ClassCart.prototype.getAddOnItemsInfo = function(addOnItem) {
	let total = 0
	addOnItem.items.forEach(p => {
		total = numAdd(total, numMultiply(p.price, p.quantity))
	})
	let diff = numSubtract(addOnItem.limitAmount, total)
	return {
		diff,
		limitAmount: addOnItem.limitAmount,
		addOnItemId: addOnItem.addOnItemId
	}
}

/**
 * 通过规则获取金额或数量
 */
ClassCart.prototype.getDiffMoney = function(rules, money, mode) {
	// 计算商品总金额
	let diffMoney = 0
	// 阶梯满减
	if (mode === REDUCTION_MODE.STEP) {
		rules.some((item, index) => {
			if (money < item.limit) {
				diffMoney = numSubtract(item.limit, money)
				return true
			}
			// 金额差为0，但是又不是最后一个，需要再次计算下一档
			if (money === item.limit && index !== rules.length - 1) {
				diffMoney = numSubtract(rules[index + 1].limit, money)
				return true
			}
			
		})
	}
	
	// 循环满减
	if (mode === REDUCTION_MODE.LOOP) {
		let rule = rules[0]
		let index = Math.ceil(numDivide(money, rule.limit))
		// 如果发现刚好是整数，向后取
		if (money % rule.limit === 0) {
			index += 1
		}
		let nextMoney = numMultiply(index, rule.limit)
		diffMoney =  numSubtract(nextMoney, money)
	}
	return diffMoney
}

const cart = new ClassCart()
export default cart
